@@ -1,7 +1,10 @@
 """The command-line parsing and entry point."""
 
 from pydoctor import model, zopeinterface
-import sys, os
+from pydoctor.sphinx import SphinxInventory
+import sys, os, datetime
+
+BUILDTIME_FORMAT = '%Y-%m-%d %H:%M:%S'
 
 def error(msg, *args):
     if args:
@@ -68,6 +71,9 @@ def getparser():
     parser.add_option(
         '--make-html', action='store_true', dest='makehtml',
         default=MAKE_HTML_DEFAULT, help=("Produce html output."))
+    parser.add_option(
+        '--make-intersphinx', action='store_true', dest='makeintersphinx',
+        default=False, help=("Produce (only) the objects.inv intersphinx file."))
     parser.add_option(
         '--server', action='store_true', dest='server',
         help=("Serve HTML on a local server."))
@@ -101,12 +107,6 @@ def getparser():
         '--prepend-package', action='store', dest='prependedpackage',
         help=("Pretend that all packages are within this one.  "
               "Can be used to document part of a package."))
-    parser.add_option(
-        '--resolve-aliases', action='store_true',
-        dest='resolvealiases', default=False,
-        help=("This updates references to classes imported from a module "
-              "into which they were imported to references to where they "
-              "are defined."))
     parser.add_option(
         '--abbreviate-specialcase', action='store',
         dest='abbrevmapping', default='',
@@ -141,28 +141,15 @@ def getparser():
     parser.add_option(
         '--html-writer', dest='htmlwriter',
         help=("Dotted name of html writer class to use (default "
-              "'pydoctor.nevowhtml.NevowWriter', requires Divmod Nevow "
-              "to be installed)."))
+              "'pydoctor.templatewriter.TemplateWriter')."))
     parser.add_option(
         '--html-viewsource-base', dest='htmlsourcebase',
         help=("This should be the path to the trac browser for the top "
               "of the svn checkout we are documenting part of."))
     parser.add_option(
-        '--html-use-sorttable', dest='htmlusesorttable',
-        default=False, action="store_true",
-        help=("Use the sorttable JS library to make tables of package, "
-              "module and class contents sortable"))
-    parser.add_option(
-        '--html-use-splitlinks', dest='htmlusesplitlinks',
-        default=False, action="store_true",
-        help=("Generate (unobstrusive) JavaScript to allow class methods to "
-              "be shown either in one table per base class or in one big "
-              "table."))
-    parser.add_option(
-        '--html-shorten-lists', dest='htmlshortenlists',
-        default=False, action="store_true",
-        help=("Generate (unobstrusive) JavaScript to hide some of the "
-              "entries in long lists of e.g. subclasses."))
+        '--buildtime', dest='buildtime',
+        help=("Use the specified build time over the current time. "
+              "Format: %s" % BUILDTIME_FORMAT))
     parser.add_option(
         '--livecheck', action='store_true', dest='livecheck',
         default=False,
@@ -192,6 +179,14 @@ def getparser():
     parser.add_option(
         '--introspect-c-modules', default=False, action='store_true',
         help=("Import and introspect any C modules found."))
+
+    parser.add_option(
+        '--intersphinx', action='append', dest='intersphinx',
+        metavar='URL_TO_OBJECTS.INV', default=[],
+        help=(
+            "Use Sphinx objects inventory to generate links to external"
+            "documetation. Can be repeated."))
+
     return parser
 
 def readConfigFile(options):
@@ -258,7 +253,10 @@ def main(args):
         else:
             system = systemclass()
 
+        # Once pickle support is removed, always instantiate System with
+        # options and make fetchIntersphinxInventories private in __init__.
         system.options = options
+        system.fetchIntersphinxInventories()
 
         system.urlprefix = ''
         if options.moresystems:
@@ -293,11 +291,28 @@ def main(args):
 
         if options.makehtml == MAKE_HTML_DEFAULT:
             if not options.outputpickle and not options.testing \
-                   and not options.server:
+                   and not options.server and not options.makeintersphinx:
                 options.makehtml = True
             else:
                 options.makehtml = False
 
+        # Support source date epoch:
+        # https://reproducible-builds.org/specs/source-date-epoch/
+        try:
+            system.buildtime = datetime.datetime.utcfromtimestamp(
+                int(os.environ['SOURCE_DATE_EPOCH']))
+        except ValueError, e:
+            error(e)
+        except KeyError:
+            pass
+
+        if options.buildtime:
+            try:
+                system.buildtime = datetime.datetime.strptime(
+                    options.buildtime, BUILDTIME_FORMAT)
+            except ValueError, e:
+                error(e)
+
         # step 2: add any packages and modules
 
         if args:
@@ -337,7 +352,9 @@ def main(args):
             system.msg(
                 'warning',
                 'WARNING: guessing '+name+' for project name', thresh=-1)
-            system.guessedprojectname = name
+            system.projectname = name
+        else:
+            system.projectname = system.options.projectname
 
         # step 4: save the system, if desired
 
@@ -352,12 +369,13 @@ def main(args):
         # step 5: make html, if desired
 
         if options.makehtml:
+            options.makeintersphinx = True
             if options.htmlwriter:
                 writerclass = findClassFromDottedName(
                     options.htmlwriter, '--html-writer')
             else:
-                from pydoctor import nevowhtml
-                writerclass = nevowhtml.NevowWriter
+                from pydoctor import templatewriter
+                writerclass = templatewriter.TemplateWriter
 
             system.msg('html', 'writing html to %s using %s.%s'%(
                 options.htmloutput, writerclass.__module__,
@@ -398,12 +416,30 @@ def main(args):
                 f.close()
                 system.options = options
 
+        if options.makeintersphinx:
+            if not options.makehtml:
+                subjects = system.rootobjects
+            # Generate Sphinx inventory.
+            sphinx_inventory = SphinxInventory(
+                logger=system.msg,
+                project_name=system.projectname,
+                )
+            if not os.path.exists(options.htmloutput):
+                os.makedirs(options.htmloutput)
+            sphinx_inventory.generate(
+                subjects=subjects,
+                basepath=options.htmloutput,
+                )
+
+
         # Finally, if we should serve html, lets serve some html.
         if options.server:
             from pydoctor.server import (
                 EditingPyDoctorResource, PyDoctorResource)
-            from pydoctor.epydoc2stan import doc2html
-            from nevow import appserver
+            from pydoctor.epydoc2stan import doc2stan
+            from twisted.web.server import Site
+            from twisted.web.resource import Resource
+            from twisted.web.vhost import VHostMonsterResource
             from twisted.internet import reactor
             if options.edit:
                 if not options.nocheck:
@@ -417,19 +453,18 @@ def main(args):
                             "server", i+1, len(included_obs),
                             "docstrings checked, found %s problems" % (
                             len(system.epytextproblems)))
-                        doc2html(ob, docstring=ob.docstring)
+                        doc2stan(ob, docstring=ob.docstring)
                 root = EditingPyDoctorResource(system)
             else:
                 root = PyDoctorResource(system)
             if options.facing_path:
                 options.local_only = True
-                from nevow import rend, vhost
-                realroot = rend.Page()
+                realroot = Resource()
                 cur = realroot
-                realroot.putChild('vhost', vhost.VHostMonsterResource())
+                realroot.putChild('vhost', VHostMonsterResource())
                 segments = options.facing_path.split('/')
                 for segment in segments[:-1]:
-                    next = rend.Page()
+                    next = Resource()
                     cur.putChild(segment, next)
                     cur = next
                 cur.putChild(segments[-1], root)
@@ -446,7 +481,7 @@ def main(args):
                 reactor.callWhenRunning(wb_open)
             from twisted.python import log
             log.startLogging(sys.stdout)
-            site = appserver.NevowSite(root)
+            site = Site(root)
             if options.local_only:
                 interface = 'localhost'
             else:
