@@ -1,12 +1,20 @@
-"""Convert epydoc markup into content renderable by Nevow."""
+"""
+Convert epydoc markup into renderable content.
+"""
 
 import __builtin__
+import exceptions
+import inspect
+import itertools
+import os
+import re
+import sys
+import urllib
+
+from twisted.web.template import Tag, tags, XMLString
 
 from pydoctor import model
 
-from nevow import tags
-
-import exceptions, inspect, itertools, os, sys, urllib
 
 STDLIB_DIR = os.path.dirname(os.__file__)
 STDLIB_URL = 'http://docs.python.org/library/'
@@ -15,6 +23,7 @@ STDLIB_URL = 'http://docs.python.org/library/'
 def link(o):
     return o.system.urlprefix + urllib.quote(o.fullName()+'.html')
 
+
 def get_parser(formatname):
     try:
         mod = __import__('epydoc.markup.' + formatname,
@@ -24,6 +33,7 @@ def get_parser(formatname):
     else:
         return mod.parse_docstring, None
 
+
 def boringDocstring(doc, summary=False):
     """Generate an HTML representation of a docstring in a really boring way.
     """
@@ -33,57 +43,147 @@ def boringDocstring(doc, summary=False):
         return '<pre class="undocumented">Undocumented</pre>'
     def crappit(): pass
     crappit.__doc__ = doc
-    return [tags.pre, tags.tt][bool(summary)][inspect.getdoc(crappit)]
+    return (tags.pre, tags.tt)[bool(summary)](inspect.getdoc(crappit))
+
+
+def stdlib_doc_link_for_name(name):
+    parts = name.split('.')
+    for i in range(len(parts), 0, -1):
+        sub_parts = parts[:i]
+        filename = '/'.join(sub_parts)
+        sub_name = '.'.join(sub_parts)
+        if sub_name == 'os.path' \
+               or os.path.exists(os.path.join(STDLIB_DIR, filename) + '.py') \
+               or os.path.exists(os.path.join(STDLIB_DIR, filename, '__init__.py')) \
+               or os.path.exists(os.path.join(STDLIB_DIR, 'lib-dynload', filename) + '.so') \
+               or sub_name in sys.builtin_module_names:
+            return STDLIB_URL + sub_name + '.html#' + name
+    part0 = parts[0]
+    if part0 in __builtin__.__dict__ and not part0.startswith('__'):
+        bltin = __builtin__.__dict__[part0]
+        if part0 in exceptions.__dict__:
+            return STDLIB_URL + 'exceptions.html#exceptions.' + name
+        elif isinstance(bltin, type):
+            return STDLIB_URL + 'stdtypes.html#' + name
+        elif callable(bltin):
+            return STDLIB_URL + 'functions.html#' + name
+        else:
+            return STDLIB_URL + 'constants.html#' + name
+    return None
+
 
 class _EpydocLinker(object):
+
     def __init__(self, obj):
         self.obj = obj
+
     def translate_indexterm(self, something):
         # X{foobar} is meant to put foobar in an index page (like, a
         # proper end-of-the-book index). Should we support that? There
         # are like 2 uses in Twisted.
         return de_p(something.to_html(self))
-    def translate_identifier_xref(self, fullID, prettyID):
-        obj = self.obj.resolveDottedName(fullID)
-        if obj is None:
-            parts = fullID.split('.')
-            linktext = None
-            for i in range(len(parts), 0, -1):
-                sub_parts = parts[:i]
-                filename = '/'.join(sub_parts)
-                sub_name = '.'.join(sub_parts)
-                if sub_name == 'os.path' \
-                       or os.path.exists(os.path.join(STDLIB_DIR, filename) + '.py') \
-                       or os.path.exists(os.path.join(STDLIB_DIR, 'lib-dynload', filename) + '.so') \
-                       or sub_name in sys.builtin_module_names:
-                    linktext = STDLIB_URL + sub_name + '.html#' + fullID
-                    break
-            else:
-                if fullID in __builtin__.__dict__:
-                    if fullID in exceptions.__dict__:
-                        linktext = STDLIB_URL + 'exceptions.html#exceptions.' + fullID
-                    else:
-                        linktext = STDLIB_URL + 'functions.html#' + fullID
-            if linktext is not None:
-                return '<a href="%s"><code>%s</code></a>'%(linktext, prettyID)
-            else:
-                self.obj.system.msg(
-                    "resolveDottedName", "%s:%s invalid ref to %s" % (
-                        self.obj.fullName(), self.obj.linenumber, fullID),
-                    thresh=-1)
-                return '<code>%s</code>'%(prettyID,)
+
+    def _objLink(self, obj, prettyID):
+        if obj.documentation_location == model.DocLocation.PARENT_PAGE:
+            p = obj.parent
+            if isinstance(p, model.Module) and p.name == '__init__':
+                p = p.parent
+            linktext = link(p) + '#' + urllib.quote(obj.name)
+        elif obj.documentation_location == model.DocLocation.OWN_PAGE:
+            linktext = link(obj)
         else:
-            if obj.documentation_location == model.DocLocation.PARENT_PAGE:
-                p = obj.parent
-                if isinstance(p, model.Module) and p.name == '__init__':
-                    p = p.parent
-                linktext = link(p) + '#' + urllib.quote(obj.name)
-            elif obj.documentation_location == model.DocLocation.OWN_PAGE:
-                linktext = link(obj)
-            else:
-                raise AssertionError(
-                    "Unknown documentation_location: %s" % obj.documentation_location)
+            raise AssertionError(
+                "Unknown documentation_location: %s" % obj.documentation_location)
+        return '<a href="%s"><code>%s</code></a>'%(linktext, prettyID)
+
+    def look_for_name(self, name, candidates):
+        part0 = name.split('.')[0]
+        potential_targets = []
+        for src in candidates:
+            if part0 not in src.contents:
+                continue
+            target = src.resolveName(name)
+            if target is not None and target not in potential_targets:
+                potential_targets.append(target)
+        if len(potential_targets) == 1:
+            return potential_targets[0]
+        elif len(potential_targets) > 1:
+            self.obj.system.msg(
+                "translate_identifier_xref", "%s:%s ambiguous ref to %s, could be %s" % (
+                    self.obj.fullName(), self.obj.linenumber, name,
+            ', '.join([ob.fullName() for ob in potential_targets])),
+                thresh=-1)
+        return None
+
+    def look_for_intersphinx(self, name):
+        """
+        Return link for `name` based on intersphinx inventory.
+
+        Return None if link is not found.
+        """
+        return self.obj.system.intersphinx.getLink(name)
+
+    def translate_identifier_xref(self, fullID, prettyID):
+        """Figure out what ``L{fullID}`` should link to.
+
+        There is a lot of DWIM here.  The order goes:
+
+          1. Check if fullID refers to an object by Python name resolution in
+             our context.
+
+          2. Walk up the object tree and see if fullID refers to an object by
+             Python name resolution in each context.
+
+          3. Check if fullID is the fullName of an object.
+
+          4. Check to see if fullID names a builtin or standard library
+             module.
+
+          4. Walk up the object tree again and see if fullID refers to an
+             object in an "uncle" object.  (So if p.m1 has a class C, the
+             docstring for p.m2 can say L{C} to refer to the class in m1).  If
+             at any level fullID refers to more than one object, complain.
+
+          5. Examine every module and package in the system and see if fullID
+             names an object in each one.  Again, if more than one object is
+             found, complain.
+
+        """
+        src = self.obj
+        while src is not None:
+            target = src.resolveName(fullID)
+            if target is not None:
+                return self._objLink(target, prettyID)
+            src = src.parent
+        target = self.obj.system.objForFullName(fullID)
+        if target is not None:
+            return self._objLink(target, prettyID)
+        fullerID = self.obj.expandName(fullID)
+        linktext = stdlib_doc_link_for_name(fullerID)
+        if linktext is not None:
             return '<a href="%s"><code>%s</code></a>'%(linktext, prettyID)
+        src = self.obj
+        while src is not None:
+            target = self.look_for_name(fullID, src.contents.values())
+            if target is not None:
+                return self._objLink(target, prettyID)
+            src = src.parent
+        target = self.look_for_name(fullID, itertools.chain(
+            self.obj.system.objectsOfType(model.Module),
+            self.obj.system.objectsOfType(model.Package)))
+        if target is not None:
+            return self._objLink(target, prettyID)
+
+        target = self.look_for_intersphinx(fullID)
+        if target:
+            return '<a href="%s"><code>%s</code></a>'%(target, prettyID)
+
+        self.obj.system.msg(
+            "translate_identifier_xref", "%s:%s invalid ref to %s" % (
+                self.obj.fullName(), self.obj.linenumber, fullID),
+            thresh=-1)
+        return '<code>%s</code>'%(prettyID,)
+
 
 class FieldDesc(object):
     def __init__(self):
@@ -105,6 +205,7 @@ class FieldDesc(object):
             contents.append("%s=%r"%(k, v))
         return "<%s(%s)>"%(self.__class__.__name__, ', '.join(contents))
 
+
 def format_desc_list(singular, descs, plural=None):
     if plural is None:
         plural = singular + 's'
@@ -119,15 +220,15 @@ def format_desc_list(singular, descs, plural=None):
     for d in descs:
         if first:
             row = tags.tr(class_="fieldStart")
-            row[tags.td(class_="fieldName")[label]]
+            row(tags.td(class_="fieldName")(label))
             first = False
         else:
             row = tags.tr()
-            row[tags.td()]
+            row(tags.td())
         if d.name is None:
-            row[tags.td(colspan=2)[d.format()]]
+            row(tags.td(colspan="2")(d.format()))
         else:
-            row[tags.td(class_="fieldArg")[d.name], tags.td[d.format()]]
+            row(tags.td(class_="fieldArg")(d.name), tags.td(d.format()))
         r.append(row)
     return r
 
@@ -145,22 +246,49 @@ def format_field_list(obj, singular, fields, plural=None):
     for field in fields:
         if first:
             row = tags.tr(class_="fieldStart")
-            row[tags.td(class_="fieldName")[label]]
+            row(tags.td(class_="fieldName")(label))
             first=False
         else:
             row = tags.tr()
-            row[tags.td()]
-        row[tags.td(colspan=2)[field.body]]
+            row(tags.td())
+        row(tags.td(colspan="2")(field.body))
         rows.append(row)
     return rows
 
+
+_ok_chars = '\r\n\t\f'
+_class = ''
+for _c in map(chr, range(0, 32)):
+    if _c not in _ok_chars:
+        _class += _c
+_control_pat = re.compile('[' + _class + ']')
+
+
+def html2stan(html):
+    """
+    Convert HTML to a Stan structure.
+
+    @param html: A HTML string.
+    @type html: L{unicode} or L{bytes}
+    """
+    if isinstance(html, unicode):
+        html = html.encode('utf8')
+
+    html = _control_pat.sub(lambda m:'\\x%02x' % ord(m.group()), html)
+    html = "<div>" + html + "</div>"
+    html = XMLString(html).load()[0].children
+    if html and html[-1] == u'\n':
+        del html[-1]
+    return html
+
+
 class Field(object):
     """Like epydoc.markup.Field, but without the gross accessor
     methods and with a formatted body."""
     def __init__(self, field, obj):
         self.tag = field.tag()
         self.arg = field.arg()
-        self.body = tags.raw(de_p(field.body().to_html(_EpydocLinker(obj))))
+        self.body = html2stan(field.body().to_html(_EpydocLinker(obj)))
 
     def __repr__(self):
         r = repr(self.body)
@@ -282,8 +410,8 @@ class FieldHandler(object):
 
         r.append(format_desc_list('Parameters', self.parameter_descs, 'Parameters'))
         if self.return_desc:
-            r.append(tags.tr(class_="fieldStart")[tags.td(class_="fieldName")['Returns'],
-                               tags.td(colspan="2")[self.return_desc.format()]])
+            r.append(tags.tr(class_="fieldStart")(tags.td(class_="fieldName")('Returns'),
+                               tags.td(colspan="2")(self.return_desc.format())))
         r.append(format_desc_list("Raises", self.raise_descs, "Raises"))
         for s, p, l in (('Author', 'Authors', self.authors),
                         ('See Also', 'See Also', self.seealsos),
@@ -303,7 +431,8 @@ class FieldHandler(object):
             label = "Unknown Field: " + fieldlist[0].kind
             r.append(format_desc_list(label, fieldlist, label))
 
-        return tags.table(class_='fieldTable')[r]
+        return tags.table(class_='fieldTable')(r)
+
 
 def de_p(s):
     if s.startswith('<p>') and s.endswith('</p>\n'):
@@ -312,6 +441,7 @@ def de_p(s):
         s = s[:-1]
     return s
 
+
 def reportErrors(obj, errs):
     for err in errs:
         if isinstance(err, str):
@@ -333,12 +463,13 @@ def reportErrors(obj, errs):
         for err in errs:
             p(err)
 
-def doc2html(obj, summary=False, docstring=None):
+
+def doc2stan(obj, summary=False, docstring=None):
     """Generate an HTML representation of a docstring"""
     if getattr(obj, 'parsed_docstring', None) is not None:
-        r = tags.raw(de_p(obj.parsed_docstring.to_html(_EpydocLinker(obj))))
+        r = html2stan(obj.parsed_docstring.to_html(_EpydocLinker(obj)))
         if getattr(obj, 'parsed_type', None) is not None:
-            r = [r, ' (type: ', tags.raw(de_p(obj.parsed_type.to_html(_EpydocLinker(obj)))), ')']
+            r = [r, ' (type: ', html2stan(obj.parsed_type.to_html(_EpydocLinker(obj))), ')']
         return r, []
     origobj = obj
     if isinstance(obj, model.Package):
@@ -373,9 +504,9 @@ def doc2html(obj, summary=False, docstring=None):
                                          plurals.get(k, k+'s')))
                 text += '; ' + ', '.join(u) + " documented"
         if summary:
-            return tags.span(class_="undocumented")[text], []
+            return tags.span(class_="undocumented")(text), []
         else:
-            return tags.div(class_="undocumented")[text], []
+            return tags.div(class_="undocumented")(text), []
     if summary:
         # Use up to three first non-empty lines of doc string as summary.
         lines = itertools.dropwhile(lambda line: not line.strip(),
@@ -383,7 +514,7 @@ def doc2html(obj, summary=False, docstring=None):
         lines = itertools.takewhile(lambda line: line.strip(), lines)
         lines = [ line.strip() for line in lines ]
         if len(lines) > 3:
-            return tags.span(class_="undocumented")['No summary'], []
+            return tags.span(class_="undocumented")('No summary'), []
         else:
             doc = ' '.join(lines)
     parse_docstring, e = get_parser(obj.system.options.docformat)
@@ -406,7 +537,7 @@ def doc2html(obj, summary=False, docstring=None):
     pdoc, fields = pdoc.split_fields()
     if pdoc is not None:
         try:
-            crap = de_p(pdoc.to_html(_EpydocLinker(source)))
+            crap = pdoc.to_html(_EpydocLinker(source))
         except Exception, e:
             reportErrors(source, [e.__class__.__name__ +': ' + str(e)])
             return (boringDocstring(doc, summary),
@@ -418,16 +549,20 @@ def doc2html(obj, summary=False, docstring=None):
     if summary:
         if not crap:
             return (), []
-        s = tags.span()[tags.raw(crap)]
+        stan = html2stan(crap)
+        if len(stan) == 1 and isinstance(stan[0], Tag) and stan[0].tagName == 'p':
+            stan = stan[0].children
+        s = tags.span(stan)
     else:
         if not crap and not fields:
             return (), []
-        s = tags.div()[tags.raw(crap)]
+        stan = html2stan(crap)
+        s = tags.div(stan)
         fh = FieldHandler(obj)
         for field in fields:
             fh.handle(Field(field, obj))
         fh.resolve_types()
-        s[fh.format()]
+        s(fh.format())
     return s, []
 
 
@@ -437,6 +572,7 @@ field_name_to_human_name = {
     'var': 'Variable',
     }
 
+
 def extract_fields(obj):
     if isinstance(obj, model.Package):
         obj = obj.contents['__init__']
